package cc.smtweb.biz.mztb.web.base.partyDataFill;

import cc.smtweb.biz.mztb.web.base.BusinessInfo;
import cc.smtweb.biz.mztb.web.base.FillOut;
import cc.smtweb.biz.mztb.web.base.FinanceInfo;
import cc.smtweb.biz.mztb.web.base.YearEndPersons;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityHelper;
import cc.smtweb.framework.core.db.cache.DictsCache;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.system.bpm.web.engine.model.listcard.lcms.LCMsService;
import cc.smtweb.framework.core.mvc.service.AbstractHandler;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

/**
 * Created by 1 at 2023-12-19 11:06:53
 * 页面【[机构数据填报编辑]的服务类
 */
@SwService
public class PartyDataFillService extends LCMsService {
    public final static String TYPE_CUSTOM = "custom";

    @Override
    public AbstractHandler createHandler(String type) {
        switch (type) {
            case TYPE_MODEL_SAVE:
                return new PartyDataFillHandler.PartyDataFillModelSaveHandler();
            case TYPE_MODEL_LOAD:
                return new PartyDataFillHandler.PartyDataFillModelLoadHandler();
            case TYPE_MODEL_LIST:
                return new PartyDataFillHandler.PartyDataFillModelListHandler();
            case TYPE_CUSTOM:
                return new PartyDataCustomHandler();
        }
        return super.createHandler(type);
    }

    /* demo
        //自定义
        public R demo(@SwBody SwMap params, UserSession us) {
            return pageHandler(params, us, TYPE_DEMO, handler -> ((DemoHandler)handler).demo());
        }
    */
    public R logicDel(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_CUSTOM, handler -> ((PartyDataCustomHandler) handler).logicDel(params, us));
    }

    public R affirm(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_CUSTOM, handler -> ((PartyDataCustomHandler) handler).affirm(params, us));
    }

    public R cancelAffirm(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_CUSTOM, handler -> ((PartyDataCustomHandler) handler).cancelAffirm(params, us));
    }

    public R pushData(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_CUSTOM, handler -> ((PartyDataCustomHandler) handler).pushData(params, us));
    }
    // 检查当前机构是否有重复任务
    public R checkRepetition(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_CUSTOM, handler -> ((PartyDataCustomHandler) handler).checkRepetition(params, us));
    }


    public R getData(@SwBody SwMap params, UserSession us){
        SwMap swMap = new SwMap();

        //1是月报  2 是年报
        int type = params.readInt("type");
        //年末数据
        int year = params.readInt("year");

        String sql ="select fo_id,fo_task\n" +
                "from "+EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME)+" a\n" +
                "where a.fo_task = (select max(fo_task) from "+EntityHelper.getSchemaTableName(FillOut.ENTITY_NAME)+" where fo_party_id=?)\n" +
                "and fo_party_id=? and fo_state <>-1";
        FillOut fillOut = DbEngine.getInstance().queryEntity(sql, FillOut.class, us.getPartyId(),us.getPartyId());
        if (EntityHelper.isEmpty(fillOut)) return R.error("未查询到上期填报数据");
        BusinessInfo businessInfo = DbEngine.getInstance().findDao(BusinessInfo.class).queryEntityWhere("bi_fo_id=?",fillOut.getId());
        if (EntityHelper.isEmpty(businessInfo)) throw new BizException("未查询到相关的业务信息!");
        businessInfo.put("bi_type_text", DictsCache.getInstance().getItemName("tbBusinessType",businessInfo.getType()));
        businessInfo.put("bi_property_text", DictsCache.getInstance().getItemName("tbNature",businessInfo.getProperty()));
        businessInfo.put("bi_initiate_text", DictsCache.getInstance().getItemName("tbSubject",businessInfo.getInitiate()));
        businessInfo.put("bi_execute_regimen_text", DictsCache.getInstance().getItemName("tbKJZD",businessInfo.getExecuteRegimen()));
        swMap.put("businessInfo",businessInfo);
        FinanceInfo financeInfo = DbEngine.getInstance().findDao(FinanceInfo.class).queryEntityWhere("fi_fo_id=?",fillOut.getId());
        if (EntityHelper.isEmpty(financeInfo)) throw new BizException("未查询到相关的财务信息!");

        swMap.put("financeInfo",financeInfo);
        if (year == 1){
            YearEndPersons yearEndPersons = DbEngine.getInstance().findDao(YearEndPersons.class).queryEntityWhere("yep_fo_id=?",fillOut.getId());
            if (EntityHelper.isEmpty(yearEndPersons)) throw new BizException("未查询到相关的年末人数信息!");
            swMap.put("yearEndPersons",yearEndPersons);
        }
        return R.success(swMap);
    }

}
